// AddConferenceDlg.cpp : implementation file
//

#include "stdafx.h"
#include "afxDialogEx.h"
#include "VaxTeleServerApp.h"
#include "AddRoomDlg.h"
#include "VaxRoom.h"
#include "StoreRoom.h"
#include "VaxServerSIP.h"	


// CAddRoomDlg dialog

IMPLEMENT_DYNAMIC(CAddRoomDlg, CPropertyPage)

CAddRoomDlg::CAddRoomDlg() : CPropertyPage(CAddRoomDlg::IDD)
{
	m_sRoomNo = _T("");
	m_sRoomName = _T("");
}

CAddRoomDlg::~CAddRoomDlg()
{
}

void CAddRoomDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_CONFERENCE_ROOM_NAME, m_ctrlComboRoomName);
	DDX_Text(pDX, IDC_EDIT_CONFERENCE_ROOM_NO, m_sRoomNo);
	DDX_CBString(pDX, IDC_COMBO_CONFERENCE_ROOM_NAME, m_sRoomName);
}

BEGIN_MESSAGE_MAP(CAddRoomDlg, CPropertyPage)
	ON_BN_CLICKED(IDC_BUTTON_CONFERENCE_ADD_ROOM, &CAddRoomDlg::OnButtonAddRoom)
	ON_BN_CLICKED(IDC_BUTTON_CONFERENCE_REMOVE_ROOM, &CAddRoomDlg::OnButtonRemoveRoom)
	ON_CBN_SELCHANGE(IDC_COMBO_CONFERENCE_ROOM_NAME, &CAddRoomDlg::OnCbnSelchangeComboConferenceRoomName)
END_MESSAGE_MAP()

// CAddRoomDlg message handlers

BOOL CAddRoomDlg::OnSetActive()
{
	AddRooms();
	ShowRoomDetails();

	return CPropertyPage::OnSetActive();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddRoomDlg::OnButtonAddRoom()
{
	UpdateData();

	if(!AddRoomChecks())
		return;

	if(m_ctrlComboRoomName.FindStringExact(0, m_sRoomName) != -1)
	{
		if(MessageBox("Room already exists, do you want to continue?", "VaxVoIP", MB_ICONEXCLAMATION|MB_YESNO) == IDNO)
			return;
	}
			
	if(!CVaxRoom::AddRoom(m_sRoomName, m_sRoomNo))
	{
		MessageBox("Room could not be added/updated. Please start the server first or can't assign same Dial-in No to multiple rooms.", "VaxVoIP", MB_OK|MB_ICONINFORMATION);
		return;
	}
	
	if(m_ctrlComboRoomName.FindStringExact(0, m_sRoomName) == -1)
		m_ctrlComboRoomName.AddString(m_sRoomName);

	MessageBox("Room added/updated successfully.", "VaxVoIP", MB_OK|MB_ICONINFORMATION);
}

void CAddRoomDlg::OnButtonRemoveRoom()
{
	UpdateData();
	
	if(m_sRoomName.IsEmpty())
	{
		MessageBox("Room Name can't be empty.", "VaxVoIP", MB_OK|MB_ICONINFORMATION);
		return;
	}

	LONG nIndex = m_ctrlComboRoomName.FindStringExact(0, m_sRoomName);
	if(nIndex == -1) return;

	CVaxRoom::RemoveRoom(m_sRoomName);
	m_ctrlComboRoomName.DeleteString(nIndex);

	ClearDlgUI();
	
	if(m_ctrlComboRoomName.GetCount() > 0)
		m_ctrlComboRoomName.SetCurSel(0);
		
	ShowRoomDetails();

	MessageBox("Room removed successfully!", "VaxVoIP", MB_OK|MB_ICONINFORMATION);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddRoomDlg::AddRooms()
{
	m_ctrlComboRoomName.ResetContent();

	CStringList objList;
	CStoreRoom::GetRoomNameAll(objList);
			
	while(!objList.IsEmpty())
	{
		m_ctrlComboRoomName.AddString(objList.RemoveHead());
	}

	if(m_ctrlComboRoomName.GetCount() > 0)
		m_ctrlComboRoomName.SetCurSel(0);
}

void CAddRoomDlg::ShowRoomDetails()
{
	LONG nSelectedIndex = m_ctrlComboRoomName.GetCurSel();
	if(nSelectedIndex == -1) return;

	CString sRoomName;
	
	m_ctrlComboRoomName.GetLBText(nSelectedIndex, sRoomName);
	if(sRoomName.IsEmpty()) return;
			
	CString sRoomNo;
	CStoreRoom::GetRoom(sRoomName, sRoomNo);
	
	m_sRoomName = sRoomName;
	m_sRoomNo = sRoomNo;

	UpdateData(FALSE);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddRoomDlg::OnCbnSelchangeComboConferenceRoomName()
{
	ShowRoomDetails();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CAddRoomDlg::AddRoomChecks()
{
	if(m_sRoomName.IsEmpty())
	{
		MessageBox("Room Name can't be empty.", "VaxVoIP", MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}

	if(m_sRoomNo.IsEmpty())
	{
		MessageBox("Room No can't be empty.", "VaxVoIP", MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}

	if(!CVaxDefine::IsDigitAll(m_sRoomNo))
	{
		MessageBox("Room No should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.", "VaxVoIP", MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}

	return TRUE;
}

void CAddRoomDlg::ClearDlgUI()
{
	m_sRoomName = "";
	m_sRoomNo = "";

	UpdateData(FALSE);
}