// CallPickUpDlg.cpp : implementation file
//

#include "stdafx.h"
#include "afxDialogEx.h"
#include "VaxTeleServerApp.h"
#include "CallPickUpDlg.h"
#include "VaxServerSIP.h"
#include "VaxCallPickUp.h"
#include "StoreUser.h"
#include "StoreCallPickUp.h"

// CCallPickUpDlg dialog

IMPLEMENT_DYNAMIC(CCallPickUpDlg, CPropertyPage)

CCallPickUpDlg::CCallPickUpDlg(): CPropertyPage(CCallPickUpDlg::IDD)
{
	m_sPickUpGroupName = _T("");
	m_sPickUpMemberName = _T("");
	m_sPickUpLoginName = _T("");
}

CCallPickUpDlg::~CCallPickUpDlg()
{
}

void CCallPickUpDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_CALL_PICKUP_GROUP, m_ctrlComboPickUpGroup);
	DDX_CBString(pDX, IDC_COMBO_CALL_PICKUP_GROUP, m_sPickUpGroupName);
	DDX_Control(pDX, IDC_LIST_PICKUP_MEMBER, m_ctrlListPickUpMember);
	DDX_Control(pDX, IDC_COMBO_LOGIN_PICKUP, m_ctrlListLogin);
	DDX_CBString(pDX, IDC_COMBO_LOGIN_PICKUP, m_sPickUpLoginName);
}


BEGIN_MESSAGE_MAP(CCallPickUpDlg, CPropertyPage)
	ON_BN_CLICKED(IDC_BUTTON_ADD_CALL_PICKUP_GROUP, &CCallPickUpDlg::OnButtonCallPickUpGroupAdd)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_CALL_PICKUP_GROUP, &CCallPickUpDlg::OnButtonCallPickUpGroupRemove)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_CALLPICKUP, &CCallPickUpDlg::OnButtonClearCallPickUpDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD_CALLPICKUP_MEMBER, &CCallPickUpDlg::OnButtonCallPickUpMemberAdd)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_CALLPICKUP_MEMBER, &CCallPickUpDlg::OnButtonCallPickUpMemberRemove)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_CALLPICKUP, &CCallPickUpDlg::OnButtonClearCallPickUpGroupDlg)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_CALLPICKUP_MEMBER, &CCallPickUpDlg::OnButtonClearCallPickUpMemberDlg)
	ON_CBN_SELCHANGE(IDC_COMBO_CALL_PICKUP_GROUP, &CCallPickUpDlg::OnCbnSelchangeComboCallPickupGroup)
END_MESSAGE_MAP()


BOOL CCallPickUpDlg::OnSetActive()
{
	AddPickUpGroups();
	AddUsers();

	ShowPickUpGroupDetails();
	
	return CPropertyPage::OnSetActive();
}

void CCallPickUpDlg::OnButtonCallPickUpGroupAdd()
{
	UpdateData();

	if(!AddPickupGroupChecks())
		return;

	if(m_ctrlComboPickUpGroup.FindStringExact(0, m_sPickUpGroupName) != -1)
	{
		if(MessageBox("Pickup Group already exists, do you want to continue?", "VaxVoIP", MB_ICONEXCLAMATION|MB_YESNO) == IDNO)
			return;
	}
		
	CStringList objMemberList;
	GetMemberNameAll(objMemberList);

	if(!CVaxCallPickUp::AddCallPickUpGroup(m_sPickUpGroupName, objMemberList))
	{
		MessageBox("Pickup Group couldn't be added.", "VaxVoIP", MB_ICONEXCLAMATION|MB_OK);
		return;
	}
		
	if(m_ctrlComboPickUpGroup.FindString(0, m_sPickUpGroupName) != -1)
		m_ctrlComboPickUpGroup.AddString(m_sPickUpGroupName);
			
	m_ctrlComboPickUpGroup.SetCurSel(0);
	m_ctrlComboPickUpGroup.GetLBText(0, m_sPickUpGroupName);

	UpdateData(FALSE);

	MessageBox("PickUp Group added/updated successfully.", "VaxVoIP", MB_ICONEXCLAMATION|MB_OK);
}

void CCallPickUpDlg::OnButtonCallPickUpGroupRemove()
{
	UpdateData();
	
	CVaxCallPickUp::RemoveCallPickUpGroup(m_sPickUpGroupName);

	LONG nIndex = m_ctrlComboPickUpGroup.FindStringExact(0, m_sPickUpGroupName);
		
	if(nIndex != -1)
		m_ctrlComboPickUpGroup.DeleteString(nIndex);

	MessageBox("PickUp Group removed successfully!", "Success", MB_ICONEXCLAMATION|MB_OK);

	ClearDlgUI();
}

void CCallPickUpDlg::OnButtonClearCallPickUpDlg()
{
	ClearDlgUI();
}

void CCallPickUpDlg::OnButtonCallPickUpMemberAdd()
{
	UpdateData();

	if(m_ctrlListPickUpMember.FindString(0, m_sPickUpLoginName) == -1)
		m_ctrlListPickUpMember.AddString(m_sPickUpLoginName);

	UpdateData(FALSE);
}

void CCallPickUpDlg::OnButtonCallPickUpMemberRemove()
{
	m_ctrlListPickUpMember.DeleteString(m_ctrlListPickUpMember.GetCurSel());

	UpdateData(FALSE);
}

void CCallPickUpDlg::OnButtonClearCallPickUpGroupDlg()
{
	m_ctrlListPickUpMember.ResetContent();
}

void CCallPickUpDlg::OnButtonClearCallPickUpMemberDlg()
{
	ClearAgentDlgContent();
}

BOOL CCallPickUpDlg::AddPickupGroupChecks()
{
	if(m_sPickUpGroupName.IsEmpty())
	{
		MessageBox("Please fill group name first.", "VaxVoIP", MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}

	return TRUE;
}

void CCallPickUpDlg::ClearDlgUI()
{
	UpdateData();

	m_sPickUpGroupName = "";
	ClearAgentDlgContent();
	
	UpdateData(FALSE);
}

void CCallPickUpDlg::ClearAgentDlgContent()
{
	m_ctrlListPickUpMember.ResetContent();

	UpdateData(FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCallPickUpDlg::GetMemberNameAll(CStringList& objList)
{
	CString sMemberName;

	for(int nCount = 0; nCount < m_ctrlListPickUpMember.GetCount(); nCount++)
	{
		m_ctrlListPickUpMember.GetText(nCount, sMemberName);
		objList.AddTail(sMemberName);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCallPickUpDlg::AddUsers()
{
	m_ctrlListLogin.ResetContent();

	CStringList objListUsers;
	CStoreUser::GetUserAll(objListUsers);

	while(!objListUsers.IsEmpty())
		m_ctrlListLogin.AddString(objListUsers.RemoveHead());
			
	if(m_ctrlListLogin.GetCount() > 0)
		m_ctrlListLogin.SetCurSel(0);
}

void CCallPickUpDlg::ShowPickUpGroupDetails()
{
	UpdateData();

	LONG nSelectedIndex = m_ctrlComboPickUpGroup.GetCurSel();
	if(nSelectedIndex == -1) return;

	CString sPickUpGroupName;
	m_ctrlComboPickUpGroup.GetLBText(nSelectedIndex, sPickUpGroupName);
	
	m_sPickUpGroupName = sPickUpGroupName;
	
	CStringList objMemberList;
	
	if(!CStoreCallPickUp::GetGroup(sPickUpGroupName, objMemberList))
		return;

	m_ctrlListPickUpMember.ResetContent();

	while(!objMemberList.IsEmpty())
		m_ctrlListPickUpMember.AddString(objMemberList.RemoveHead());

	UpdateData(FALSE);
}

void CCallPickUpDlg::AddPickUpGroups()
{
	UpdateData();

	m_ctrlComboPickUpGroup.ResetContent();
	
	CStringList objList;
	CStoreCallPickUp::GetGroupNameAll(objList);

	while(!objList.IsEmpty())
		m_ctrlComboPickUpGroup.AddString(objList.RemoveHead());

	if(m_ctrlComboPickUpGroup.GetCount() > 0)
		m_ctrlComboPickUpGroup.SetCurSel(0);
}

void CCallPickUpDlg::OnCbnSelchangeComboCallPickupGroup()
{
	ShowPickUpGroupDetails();
}
