// GeneralDlg.cpp : implementation file
//

#include "stdafx.h"
#include "VaxTeleServerApp.h"
#include "VaxTeleServerAppDlg.h"
#include "GeneralDlg.h"
#include "VaxServerSIP.h"
#include "StoreCOMM.h"
#include "CPU.h"
#include "VaxUser.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDC_STATS_TIMER	9005

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

#define STATUS_MSG_SERVER_RUNNING		"SIP SERVER IS RUNNING."
#define STATUS_MSG_SERVER_NOT_RUNNING	"SIP SERVER IS NOT RUNNING.";

#define START_BUTTON_TITLE	"Start"
#define STOP_BUTTON_TITLE	"Stop"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CGeneralDlg::CGeneralDlg() : CPropertyPage(CGeneralDlg::IDD)
{
	//{{AFX_DATA_INIT(CGeneralDlg)
	
	m_sHoldPlayWaveFile = _T("");
	m_sDialTonePlayWaveFile = _T("");
	m_sPlayWaveParkFile = _T("");
	
	m_sServerStatus = _T("");
	m_nListenPort = SIP_DEFAULT_PORT;

	//}}AFX_DATA_INIT

	m_psp.dwFlags &= ~PSP_HASHELP;  // Lose the Help button

	m_bServerRunning = FALSE;

}


void CGeneralDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGeneralDlg)
	DDX_Text(pDX, IDC_EDIT_SIP_SERVER_HOLD_MUSIC, m_sHoldPlayWaveFile);
	DDX_Text(pDX, IDC_EDIT_SIP_SERVER_DIAL_TONE_MUSIC, m_sDialTonePlayWaveFile);

	DDX_Text(pDX, IDC_EDIT_SIP_SERVER_LISTEN_PORT, m_nListenPort);
	DDX_Text(pDX, IDC_STATIC_SERVER_STATUS, m_sServerStatus);
	//}}AFX_DATA_MAP
	DDX_Text(pDX, IDC_EDIT_SIP_SERVER_PARK_MUSIC, m_sPlayWaveParkFile);
	DDX_Control(pDX, IDC_BUTTON_SERVER_START, m_ctrlBtnStart);
}


BEGIN_MESSAGE_MAP(CGeneralDlg, CPropertyPage)
	//{{AFX_MSG_MAP(CGeneralDlg)
	ON_BN_CLICKED(IDC_BUTTON_SERVER_START, OnButtonServerStart)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_WM_DESTROY()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGeneralDlg message handlers


BOOL CGeneralDlg::OnSetActive() 
{
	//MessageBox("GeneralDlg - OnSetActivePage", "", MB_OK);

	return CPropertyPage::OnSetActive();
}

BOOL CGeneralDlg::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	LoadCOMM();
		
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CGeneralDlg::LoadCOMM()
{
	LONG nListenPortSIP = CStoreCOMM::GetListenPortSIP();
	m_nListenPort = nListenPortSIP;

	CStoreCOMM::GetMusicFiles(m_sHoldPlayWaveFile, m_sDialTonePlayWaveFile, m_sPlayWaveParkFile);
	UpdateData(FALSE);
}

void CGeneralDlg::OnButtonServerStart() 
{
	UpdateData();

	if (m_bServerRunning)
	{
		KillTimer(IDC_STATS_TIMER);
		CVaxServerSIP::m_pVaxSIP->StopServer();

		m_sServerStatus = STATUS_MSG_SERVER_NOT_RUNNING;
		m_ctrlBtnStart.SetWindowText(START_BUTTON_TITLE);
		
		m_bServerRunning = FALSE;
		UpdateData(FALSE);

		return;
	}
		
	BOOL bResult = CVaxServerSIP::m_pVaxSIP->StartServer(m_nListenPort);
	
	if(!bResult)
	{
		m_sServerStatus = "UNABLE TO START SIP SERVER (SEE ERROR LOG).";
		UpdateData(FALSE);

		return;
	}

	CStoreCOMM::SetListenPortSIP(m_nListenPort);
	CVaxServerSIP::m_pVaxSIP->LoadMusicFiles(m_sHoldPlayWaveFile, m_sDialTonePlayWaveFile, m_sPlayWaveParkFile);

	m_sServerStatus = STATUS_MSG_SERVER_RUNNING;
	m_ctrlBtnStart.SetWindowText(STOP_BUTTON_TITLE);
		
	SetTimer(IDC_STATS_TIMER, 2000, NULL);

	m_bServerRunning = TRUE;
	UpdateData(FALSE);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////

void CGeneralDlg::OnTimer(UINT_PTR nIDEvent) 
{
	UpdateStats();
	CPropertyPage::OnTimer(nIDEvent);
}

void CGeneralDlg::UpdateStats()
{
	CString sCount;
	CWnd* pWnd;
	
	sCount.Format("%d", CVaxServerSIP::m_pVaxSIP->GetCallCount());
	pWnd = GetDlgItem(IDC_STATIC_TOTAL_CALLS);
	if(pWnd) pWnd->SetWindowText(sCount);
	
	sCount.Format("%d", CVaxUser::GetRegisteredUserCount());
	pWnd = GetDlgItem(IDC_STATIC_TOTAL_USERS);
	if(pWnd) pWnd->SetWindowText(sCount);
	
	sCount.Format("%d", GetUsageCPU());
	pWnd = GetDlgItem(IDC_STATIC_USAGE_CPU);
	if(pWnd) pWnd->SetWindowText(sCount);
	
	sCount.Format("%d MB", GetAvailableMEM());
	pWnd = GetDlgItem(IDC_STATIC_AVAIL_MEME);
	if(pWnd) pWnd->SetWindowText(sCount);

}

void CGeneralDlg::ClearStats()
{
	CWnd* pWnd;
	
	pWnd = GetDlgItem(IDC_STATIC_TOTAL_CALLS);
	pWnd->SetWindowTextA("0");

	pWnd = GetDlgItem(IDC_STATIC_TOTAL_USERS);
	pWnd->SetWindowTextA("0");
}

DWORD CGeneralDlg::GetAvailableMEM()
{
	MEMORYSTATUSEX hStatex;
	hStatex.dwLength = sizeof (hStatex);

	GlobalMemoryStatusEx (&hStatex);	// memory retrieved in bytes

	return (DWORD) hStatex.ullAvailPhys / (1024 * 1024);	// return in the form of mega bytes.
}

LONG CGeneralDlg::GetUsageCPU()
{
	CPU objCPU;
	return objCPU.GetUsageCPU();
}

void CGeneralDlg::OnDestroy()
{
	CPropertyPage::OnDestroy();
}
