#include "StdAfx.h"
#include "VaxCallSessionRoute.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVaxCallSessionRoute::CVaxCallSessionRoute()
{
	m_pUserRouteList = new CVaxHashText(64);
	m_pLineRouteList = new CVaxHashText(64);
}

CVaxCallSessionRoute::~CVaxCallSessionRoute()
{
	delete m_pUserRouteList;
	delete m_pLineRouteList;
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

CUserRouteDetails* CVaxCallSessionRoute::FindUserRoute(LPCTSTR sUserName)
{
	return (CUserRouteDetails*) m_pUserRouteList->FindData(sUserName);
}

CLineRouteDetails* CVaxCallSessionRoute::FindLineRoute(LPCTSTR sLineName)
{
	return (CLineRouteDetails*) m_pLineRouteList->FindData(sLineName);
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

BOOL CVaxCallSessionRoute::GetUserRouteDetails(LPCTSTR sUserName, CString& sUserPrefix, CString& sDialPrefix, CString& sToPeerName)
{
	BOOL bResult = FALSE;
	
	CUserRouteDetails* pDetails = FindUserRoute(sUserName);
	if(pDetails == NULL) return FALSE;

	sUserPrefix = pDetails->m_sUserPrefix;
	sDialPrefix = pDetails->m_sDialPrefix;
	sToPeerName = pDetails->m_sToPeerName;

	return TRUE;
}

BOOL CVaxCallSessionRoute::GetLineRouteDetails(LPCTSTR sLineName, CString& sToPeerName, BOOL& bRejected, CString& sRoomName, CString& sQueueName, CString& sRingGroupName)
{
	CLineRouteDetails* pDetails = FindLineRoute(sLineName);
	if(pDetails == NULL) return FALSE;

	sRoomName      = pDetails->m_sRoomName;
	sToPeerName    = pDetails->m_sToPeerName;
	bRejected      = pDetails->m_bRejected;

	sQueueName	   = pDetails->m_sQueueName;
	sRingGroupName = pDetails->m_sRingGroupName;
	
	return TRUE;
}

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

void CVaxCallSessionRoute::AddUserRoute(LPCTSTR sUserName, LPCTSTR sUserPrefix, LPCTSTR sDialPrefix, LPCTSTR sToPeerName)
{
	RemoveUserRoute(sUserName);
		
	CUserRouteDetails* pDetails = new CUserRouteDetails();
		
	pDetails->m_sUserPrefix = sUserPrefix;
	pDetails->m_sDialPrefix = sDialPrefix;
	pDetails->m_sToPeerName = sToPeerName;
		
	m_pUserRouteList->AddData(sUserName, pDetails);
}

void CVaxCallSessionRoute::RemoveUserRoute(LPCTSTR sUserName)
{
	m_pUserRouteList->DeleteData(sUserName);
}

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

void CVaxCallSessionRoute::AddLineRoute(LPCTSTR sLineName, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName, LPCTSTR sQueueName, LPCTSTR sRingGroupName)
{
	RemoveLineRoute(sLineName);

	CLineRouteDetails* pDetails = new CLineRouteDetails();
			
	pDetails->m_sToPeerName = sToPeerName;

	pDetails->m_bRejected = bRejected;
	pDetails->m_sRoomName = sRoomName;
			
	pDetails->m_sQueueName = sQueueName;
	pDetails->m_sRingGroupName = sRingGroupName;
		
	m_pLineRouteList->AddData(sLineName, pDetails);
}

void CVaxCallSessionRoute::RemoveLineRoute(LPCTSTR sLineName)
{
	m_pLineRouteList->DeleteData(sLineName);
}

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

BOOL CVaxCallSessionRoute::GetUserRouteInfo(LPCTSTR sUserName, LPCTSTR sDialNo, CString& sDialToNo, CString& sToPeerName)
{
	CString sUserPrefix, sDialPrefix;
	
	BOOL bResult = GetUserRouteDetails(sUserName, sUserPrefix, sDialPrefix, sToPeerName);
	
	// USER-ROUTE not found. Make it U2U call.
	
	if(!bResult || sToPeerName.IsEmpty() || sUserPrefix.IsEmpty())
	{
		sDialToNo = sDialNo;
		sToPeerName = sDialNo;

		return TRUE;
	}

	LONG nUserPrefixLength = sUserPrefix.GetLength();
	
	// Check if first 'N' charachters are equal to sUserPrefix. If YES; Modify it according to dialplan.
	
	CString sCallTo = sDialNo;

	if(sCallTo.Left(nUserPrefixLength) == sUserPrefix )
	{
		sCallTo.Delete(0, nUserPrefixLength);
		sDialToNo.Format("%s%s", sDialPrefix, sCallTo);
	}
	else
	{
		//Make it USER_TO_USER call.

		sDialToNo = sDialNo;
		sToPeerName = sDialNo;
	}

	return TRUE;
}

void CVaxCallSessionRoute::GetLineRouteInfo(LPCTSTR sLineName, LPCTSTR sDialNo, CString& sDialToNo, CString& sToPeerName, BOOL& bRejected, CString& sRoomName, CString& sQueueName, CString& sRingGroupName)
{
	BOOL bResult = GetLineRouteDetails(sLineName, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);
	
	if(bResult == TRUE) 
	{
		if(CString(sToPeerName).IsEmpty() == FALSE)
			sDialToNo = sToPeerName;

		return;
	}

	sDialToNo = sDialNo;
	sToPeerName = sDialNo;
}