#include "stdafx.h"
#include "VaxServerSIP.h"
#include "VaxQueue.h"
#include "VaxQueueList.h"
#include "StoreQueue.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

CVaxQueueList* CVaxQueue::m_pQueueList = NULL;

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

CVaxQueue::CVaxQueue(void)
{
	if(CVaxQueue::m_pQueueList == NULL)
		CVaxQueue::m_pQueueList = new CVaxQueueList();
}

CVaxQueue::~CVaxQueue(void)
{
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxQueue::ResetAll(BOOL bAlloc, BOOL bDeAlloc)
{
	if(bDeAlloc)
	{
		if(CVaxQueue::m_pQueueList != NULL)
		{
			delete CVaxQueue::m_pQueueList;
			CVaxQueue::m_pQueueList = NULL;
		}
	}

	if(bAlloc)
		CVaxQueue::m_pQueueList = new CVaxQueueList();
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxQueue::AddQueueList(LPCTSTR sQueueName, LPCTSTR sMusicFileName, LPCTSTR sDialToneFileName)
{
	CVaxQueue::RemoveQueue(sQueueName);

	LONG nMusicWaveId = CVaxServerSIP::m_pVaxSIP->LoadWaveFile(sMusicFileName);
	LONG nDialToneWaveId = CVaxServerSIP::m_pVaxSIP->LoadWaveFile(sDialToneFileName);

	CVaxQueue::m_pQueueList->AddQueue(sQueueName, nMusicWaveId, nDialToneWaveId);
}

void CVaxQueue::RemoveQueueList(LPCTSTR sQueueName)
{
	LONG nMusicWaveId = CVaxQueue::m_pQueueList->GetQueueMusicId(sQueueName);
	LONG nDialToneWaveId = CVaxQueue::m_pQueueList->GetQueueDialToneId(sQueueName);

	CVaxServerSIP::m_pVaxSIP->UnLoadWaveID(nMusicWaveId);
    CVaxServerSIP::m_pVaxSIP->UnLoadWaveID(nDialToneWaveId);

    CVaxQueue::m_pQueueList->RemoveQueue(sQueueName);
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxQueue::OnIncomingCall(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sQueueName, BOOL& bProcessed)
{
	bProcessed = FALSE;
	
	if(CString(sQueueName).IsEmpty())
		return;
		
	if(!CVaxServerSIP::m_pVaxSIP->AddCallSessionToQueue(sQueueName, sCallerName, sCallerId, nSessionId, 20))
		return;

	bProcessed = TRUE;
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxQueue::OnQueuePlayWaveStarted(LONG nEventType, LPCTSTR sQueueName, LONG nSessionId)
{
	if(nEventType == VAX_QUEUE_PLAY_WAVE_EVENT_TYPE_MUSIC)
	{
		LONG nQueueMusicId = CVaxQueue::m_pQueueList->GetQueueMusicId(sQueueName);
		if(nQueueMusicId == -1) return;

		CVaxServerSIP::m_pVaxSIP->PlayWaveStartToCallSession(nSessionId, VAX_CALL_CHANNEL_ZERO, nQueueMusicId, TRUE, 0);
		return;
	}

	LONG nQueueDialToneId = CVaxQueue::m_pQueueList->GetQueueDialToneId(sQueueName);
	if(nQueueDialToneId == -1) return;

	CVaxServerSIP::m_pVaxSIP->PlayWaveStartToCallSession(nSessionId, VAX_CALL_CHANNEL_ZERO, nQueueDialToneId, TRUE, 0);
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxQueue::OnQueuePlayWaveEnded(LONG nEventType, LPCTSTR sQueueName, LONG nSessionId)
{
	CVaxServerSIP::m_pVaxSIP->PlayWaveStopToCallSession(nSessionId, -1, -1);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxQueue::AddQueue(LPCTSTR sQueueName, LONG nProcessMode, LONG nProcessType, CStringList& objAgentList, LPCTSTR sMusicFileName, LPCTSTR sDialToneFileName)
{
	if(!CVaxServerSIP::m_pVaxSIP->IsInitialized() || CString(sQueueName).IsEmpty())
		return FALSE;
	
	RemoveQueue(sQueueName);

	if (!CVaxServerSIP::m_pVaxSIP->AddQueue(sQueueName))
		return FALSE;

	POSITION hPos = objAgentList.GetHeadPosition();

	while(hPos != NULL)
	{
		CString sAgentName = objAgentList.GetNext(hPos);
		CVaxServerSIP::m_pVaxSIP->AddQueueAgent(sQueueName, sAgentName);
	}

	CVaxServerSIP::m_pVaxSIP->SetQueueProcessMode(sQueueName, nProcessMode, nProcessType);
    AddQueueList(sQueueName, sMusicFileName, sDialToneFileName);

	CStoreQueue::AddQueue(sQueueName, nProcessMode, nProcessType, objAgentList, sMusicFileName, sDialToneFileName);
	return TRUE;
}

void CVaxQueue::RemoveQueue(LPCTSTR sQueueName)
{
	if(!CVaxServerSIP::m_pVaxSIP->IsInitialized() || CString(sQueueName).IsEmpty())
		return;

	RemoveQueueList(sQueueName);
	CVaxServerSIP::m_pVaxSIP->RemoveQueue(sQueueName);
	CStoreQueue::RemoveQueue(sQueueName);
}

void CVaxQueue::LoadQueue()
{
	if(!CVaxServerSIP::m_pVaxSIP->IsInitialized())
		return;

	CStringList objListQueue;
	CStoreQueue::GetQueueNameAll(objListQueue);

	if(objListQueue.IsEmpty())
		return;

	CStringList objAgentList;
	
	CString sQueueName, sMusicFileName, sDialToneFileName;
	LONG nProcessMode = -1, nProcessType = -1;
		
	while(!objListQueue.IsEmpty())
	{
		sQueueName = objListQueue.RemoveHead();
	   
		CStoreQueue::GetQueue(sQueueName, nProcessMode, nProcessType, objAgentList, sMusicFileName, sDialToneFileName);
		AddQueue(sQueueName, nProcessMode, nProcessType, objAgentList, sMusicFileName, sDialToneFileName);
   }
}