#include "StdAfx.h"
#include "Define.h"

#include "StatusLogDlg.h"
#include "VaxServerSIP.h"
#include "VaxCallSession.h"
#include "VaxCallSessionList.h"

#include "VaxQueue.h"
#include "VaxRingGroup.h"

#include "VaxCallParking.h"
#include "VaxCallPickUp.h"
#include "VaxStealthListen.h"

#include "VaxUser.h"
#include "VaxLine.h"
#include "VaxRoom.h"

#include "StoreCOMM.h"
#include "VaxSettingDefault.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVaxServerSIP* CVaxServerSIP::m_pVaxSIP = NULL;

CVaxServerSIP::CVaxServerSIP()
{
	CVaxServerSIP::m_pVaxSIP = this;
		
	m_bInitialized = FALSE;
	ResetAll(TRUE, FALSE);
}

CVaxServerSIP::~CVaxServerSIP()
{
	UnInitialize();
	
	CVaxServerCOM::UnLoadObjectCOM();
	ResetAll(FALSE, TRUE);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::ResetAll(BOOL bAlloc, BOOL bDeAlloc)
{
	m_nDialToneId = -1;
	
	if(bDeAlloc)
	{
		delete m_pSessionList;
		m_pSessionList = NULL;
	}
		
	if(bAlloc)
	{
		m_pSessionList = new CVaxCallSessionList();
	}
	
	CVaxUser::ResetAll(bAlloc, bDeAlloc);
	CVaxCallSession::ResetAll(bAlloc, bDeAlloc);
	CVaxQueue::ResetAll(bAlloc, bDeAlloc);

	CVaxCallParking::ResetAll(bAlloc, bDeAlloc);
	CVaxStealthListen::ResetAll(bAlloc, bDeAlloc);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerSIP::StartServer(LONG nListenPort)
{
	StopServer();

	CVaxServerCOM::SetLicenseKey("TRIAL-LICENSE-KEY");

	if(!CVaxServerCOM::Initialize(""))
		return FALSE;

	if(!CVaxServerCOM::OpenNetworkUDP("", nListenPort))
	{
		UnInitialize();
		return FALSE;
	}

	if(!CVaxServerCOM::OpenNetworkTCP("", nListenPort))
    {
		UnInitialize();
		return FALSE;
	}

	if(!CVaxServerCOM::OpenNetworkTLS("", nListenPort + 1, ""))
	{
		UnInitialize();
		return FALSE;
	}

	CVaxServerCOM::SetListenPortRangeRTP(20000, 30000);
    CVaxServerCOM::AudioSessionLost(TRUE, 30); // Enable voice session lost for 30 seconds.
    CVaxServerCOM::VideoCOMM(TRUE);

	CVaxServerCOM::AddLine("AnonymousUDP", VAX_LINE_TYPE_UDP, "", "", "", "", "", "255.255.255.255", -1, "");
	CVaxServerCOM::AddLine("AnonymousTCP", VAX_LINE_TYPE_TCP, "", "", "", "", "", "255.255.255.255", -1, "");
	CVaxServerCOM::AddLine("AnonymousTLS", VAX_LINE_TYPE_TLS, "", "", "", "", "", "255.255.255.255", -1, "");

	CVaxServerCOM::AddCallPickUpGroupRouteUID("#5#"); // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.
	CVaxServerCOM::AddStealthListenRouteUID("#1#");   // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.
	CVaxServerCOM::AddCallParkRouteUID("#3#");        // OnIncomingCall(), OnCallSessionTransferBlind(), sRouteUID Parameter.
	
	//CVaxServerCOM::ActivatePushSIP(true, 1);

	m_bInitialized = true;
	OnInitialize();
   
	return TRUE;
}

void CVaxServerSIP::StopServer()
{
	m_bInitialized = FALSE;
				
	CVaxServerCOM::UnInitialize();
	ResetAll(TRUE, TRUE);
}

BOOL CVaxServerSIP::IsInitialized()
{
	return m_bInitialized;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerSIP::AcceptCallSession(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sDialNo, LPCTSTR sToPeerName, LPCTSTR sRouteUID)
{
	return CVaxServerCOM::AcceptCallSession(nSessionId, sCallerName, sCallerId, sDialNo, sToPeerName, sRouteUID, 30);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerSIP::DialToneToCallSession(BOOL bEnable, LONG nSessionId)
{
	if(m_nDialToneId == -1) return TRUE;
	return CVaxServerCOM::DialToneToCallSession(bEnable, nSessionId, m_nDialToneId);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerSIP::LoadMusicDialTone(LPCTSTR sFileName)
{
	if (m_nDialToneId != -1)
	{
		CVaxServerCOM::UnLoadWaveID(m_nDialToneId);
		m_nDialToneId = -1;
	}

    m_nDialToneId = CVaxServerCOM::LoadWaveFile(sFileName);
	return (m_nDialToneId != -1);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::LoadMusicFiles(LPCTSTR sMusicHold, LPCTSTR sMusicDialTone, LPCTSTR sMusicCallPark)
{
	CVaxServerCOM::LoadMusicHold(sMusicHold);
    LoadMusicDialTone(sMusicDialTone);

    CVaxCallParking::LoadMusic(sMusicCallPark);
	CVaxStealthListen::LoadMusic(sMusicCallPark);

    CStoreCOMM::SetMusicFiles(sMusicHold, sMusicDialTone, sMusicCallPark);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnCallSessionCreated(LONG nSessionId, LONG nReasonCode)
{
	CVaxCallSession* pCallSession = new CVaxCallSession(); 
	m_pSessionList->AddSession(nSessionId, pCallSession);
}

void CVaxServerSIP::OnCallSessionClosed(LONG nSessionId, LONG nChannelId, LONG nReasonCode)
{
	m_pSessionList->RemoveSession(nSessionId);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnCallSessionPushSIP(LONG nSessionId, LONG nChannelId, LPCTSTR sCallerName, LPCTSTR sCallerId, LONG nProviderPN, LPCTSTR sProjectPN, LPCTSTR sDevicePN)
{
	// Use OnCallSessionPushSIP() event to send Push-Notification to Android and iOS based devices. 
	// OnCallSessionPushSIP() event triggers only if softphone or SIP client supports Push-SIP RFC.
    // VaxVoIP Phone SDK iOS & Android already support Push-SIP RFC.

	//CVaxServerCOM::ActivatePushSIP() VaxServerSDK Exported method. Please search in this Class. Please use it in StartServer()  function.
       
	/*
	if(nProviderPN == VAX_PUSH_PROVIDER_IOS)
	{
		Send Notification by using HTTP, Curl.exe or any other method to iOS device. sDevicePN is device token. sProjectPN is APP Name.
	}
	else if (nProviderPN == VAX_PUSH_PROVIDER_ANDROID)
	{
		Send Notification by using HTTP, Curl.exe or any other method to Android device. sDevicePN is device token. sProjectPN is APP Name.
	}
	*/
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnIncomingCall(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sDialNo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sRouteUID, LONG nRouteType, LPCTSTR sRouteName, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort)
{
	if(!CString(sRouteUID).IsEmpty())
	{
		if(nRouteType == VAX_ROUTE_TYPE_CALL_PARK)
		{
			CVaxCallParking::OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, sRouteUID);
			return;
		}

		if (nRouteType == VAX_ROUTE_TYPE_STEALTH_LISTEN)
		{
			CVaxStealthListen::OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sRouteUID);
            return;
		}

		AcceptCallSession(nSessionId, sCallerName, sCallerId, sDialNo, "", sRouteUID);
		return;
	}

	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	if(pCallSession == NULL) return;
		
	DialToneToCallSession(TRUE, nSessionId);
	pCallSession->OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sUserAgent, sFromIP, nFromPort);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnRegisterUser(LPCTSTR sUserLogin, LPCTSTR sDomain, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser::OnRegisterUser(sUserLogin, sDomain, sUserAgent, sFromIP, nFromPort, nRegId);
}

void CVaxServerSIP::OnRegisterUserSuccess(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser::OnRegisterUserSuccess(sUserLogin, sFromIP, nFromPort, nRegId);
}

void CVaxServerSIP::OnRegisterUserFailed(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser::OnRegisterUserFailed(sUserLogin, sFromIP, nFromPort, nRegId);
}

void CVaxServerSIP::OnUnRegisterUser(LPCTSTR sUserLogin)
{
	CVaxUser::OnUnRegisterUser(sUserLogin);
}

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnCallSessionTransferBlind(LONG nTransfererSessionId, LONG nTransfererChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo, LPCTSTR sRouteUID, LONG nRouteType, LPCTSTR sRouteName)
{
	if(!CString(sRouteUID).IsEmpty())
	{
		if(nRouteType == VAX_ROUTE_TYPE_CALL_PARK)
		{
			CVaxCallParking::OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferee, sTransferTo, sRouteUID);
			return;
		}

		CVaxServerCOM::AcceptTransferBlind(nTransfererSessionId, sTransferee, sTransferee, sTransferTo, "", sRouteUID, 30);
		return;
	}

	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nTransfererSessionId);
	pCallSession->OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);
}

void CVaxServerSIP::OnCallSessionTransferConsult(LONG nTransfererSessionId, LONG nTransfereeSessionId, LONG nTransferToSessionId, LONG nTransfererChannelId, LONG nTransfereeChannelId, LONG nTransferToChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nTransfererSessionId);
	pCallSession->OnCallSessionTransferConsult(nTransfererSessionId, nTransfererChannelId, nTransferToSessionId, nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnChatMessageText(LONG nChatMsgId, LPCTSTR sMsgFrom, LPCTSTR sMsgTo, LPCTSTR sMsgText, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sFromIP, LONG nFromPort)
{
	CVaxServerCOM::AcceptChatMessage(nChatMsgId, sMsgTo);
}

void CVaxServerSIP::OnChatMessageTyping(LONG nChatMsgId, LPCTSTR sMsgFrom, LPCTSTR sMsgTo, BOOL bIsTypingStart, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sFromIP, LONG nFromPort)
{
	CVaxServerCOM::AcceptChatMessage(nChatMsgId, sMsgTo);
}

void CVaxServerSIP::OnChatStatusSubscribe(LONG nSubscrbId, LPCTSTR sMsgFrom, LPCTSTR sMsgTo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sFromIP, LONG nFromPort)
{
	CVaxServerCOM::AcceptChatStatusSubscribe(nSubscrbId, sMsgFrom, sMsgTo, sMsgTo);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnBusyLampSubscribe(LONG nSubscrbId, LPCTSTR sUserName, LPCTSTR sToUserName, LPCTSTR sFromIP, LONG nFromPort)
{
	CVaxServerCOM::BusyLampSubscribeAccept(nSubscrbId, TRUE, 30);
}

void CVaxServerSIP::OnBusyLampSendStatus(LPCTSTR sFromUserName, LPCTSTR sToUserName, LONG nStateId)
{
	CVaxServerCOM::BusyLampSendStatus(sFromUserName, sToUserName, nStateId);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnLineRegisterTrying(LPCTSTR sLineName)
{
	CVaxLine::OnLineRegisterTrying(sLineName);
}

void CVaxServerSIP::OnLineRegisterFailed(LPCTSTR sLineName, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
	CVaxLine::OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);
}

void CVaxServerSIP::OnLineRegisterSuccess(LPCTSTR sLineName)
{
	CVaxLine::OnLineRegisterSuccess(sLineName);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnDetectedDigitDTMF(LONG nSessionId, LONG nChannelId, LPCTSTR sDigitDTMF, int nTypeDTMF)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	pCallSession->OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnCallSessionDetectedAMD(LONG nSessionId, LONG nChannelId, BOOL bIsHuman)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	pCallSession->OnCallSessionDetectedAMD(nSessionId, nChannelId, bIsHuman);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnQueuePlayWaveStarted(LONG nEventType, LPCTSTR sQueueName, LONG nSessionId)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	pCallSession->OnQueuePlayWaveStarted(nEventType, sQueueName, nSessionId);
}

void  CVaxServerSIP::OnQueuePlayWaveEnded(LONG nEventType, LPCTSTR sQueueName, LONG nSessionId)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	pCallSession->OnQueuePlayWaveEnded(nEventType, sQueueName, nSessionId);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnStealthListenerAttached(LONG nAttachType, LONG nStealthSessionId, LONG nAttachSessionId, LONG nAttachChannelId)
{
	CVaxStealthListen::OnStealthListenerAttached(nAttachType, nStealthSessionId, nAttachSessionId, nAttachChannelId);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::GenStatusLog(LPCTSTR sStatus)
{
	OnStatusLog(sStatus);
}

void CVaxServerSIP::OnVaxErrorLog(LPCTSTR sFuncName, LONG nErrorCode, LPCTSTR sErrorMsg)
{
	CString sMsg;
	sMsg.Format("[Method: %s] [Error Msg: %s] [Code: %d]", sFuncName, sErrorMsg, nErrorCode);

	OnStatusLog(sMsg);
}

void CVaxServerSIP::OnCallSessionErrorLog(LONG nSessionId, LONG nChannelId, LONG nErrorCode, LPCTSTR sErrorMsg)
{
	CString sMsg;
	sMsg.Format("[Session: %d] [Error Msg: %s] [Code: %d]", nSessionId, sErrorMsg, nErrorCode);

	OnStatusLog(sMsg);
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

void CVaxServerSIP::OnInitialize()
{
	if(!CVaxSettingDefault::LoadUserDefault())
	{
		CVaxUser::LoadUser();
	}
	
	if(!CVaxSettingDefault::LoadLineDefault())
	{
		CVaxLine::LoadLine();
	}
	
	if(!CVaxSettingDefault::LoadRoomDefault())
	{
		CVaxRoom::LoadRoom();
	}
    
	if(!CVaxSettingDefault::LoadQueueDefault())
	{
		CVaxQueue::LoadQueue();
	}

	if(!CVaxSettingDefault::LoadRingGroupDefault())
	{
		CVaxRingGroup::LoadRingGroup();
	}

	if(!CVaxSettingDefault::LoadCallPickUpGroupDefault())
	{
		CVaxCallPickUp::LoadCallPickupGroup();
	}
}

void CVaxServerSIP::OnStatusLog(LPCTSTR sStatus)
{
	m_pStatusLogDlg->OnStatusLog(sStatus);
}