#include "stdafx.h"
#include "VaxServerSIP.h"

#include "VaxUserList.h"
#include "VaxUser.h"

#include "StoreUser.h"
#include "VaxCallSession.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVaxUserList* CVaxUser::m_pUserList = NULL;
LONG CVaxUser::m_nRegisterUserCount = 0;

CVaxUser::CVaxUser(LPCTSTR sUserName, BOOL bBlocked, LONG nVoiceTypeId)
{
	if(CVaxUser::m_pUserList == NULL)
		CVaxUser::m_pUserList = new CVaxUserList();
	   	
	m_bBlocked = bBlocked;
	m_bRegistered = FALSE;

	m_nVoiceTypeId = nVoiceTypeId;
}

CVaxUser::~CVaxUser(void)
{
}


void CVaxUser::ResetAll(BOOL bAlloc, BOOL bDeAlloc)
{
	m_nRegisterUserCount = 0;
	
	if(bDeAlloc)
	{
		if(CVaxUser::m_pUserList != NULL)
		{
			delete CVaxUser::m_pUserList;
			CVaxUser::m_pUserList = NULL;
		}
	}

	if(bAlloc)
		CVaxUser::m_pUserList = new CVaxUserList();;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

CVaxUser* CVaxUser::GetUser(LPCTSTR sUserName)
{
	return m_pUserList->GetUser(sUserName);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnRegisterUser(LPCTSTR sUserLogin, LPCTSTR sDomain, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserLogin);
	
	if(pUser == NULL)
	{
		CVaxServerSIP::m_pVaxSIP->RejectRegister(nRegId, 404, "Not Found");

		CString sStatus = sUserLogin + CString(" ") + USER_NOT_FOUND;
		CVaxServerSIP::m_pVaxSIP->GenStatusLog(sStatus);

		return;
	}
				
	pUser->OnRegister(sUserLogin, nRegId);
}

void CVaxUser::OnRegister(LPCTSTR sUserLogin, LONG nRegId)
{
	if (m_bBlocked)
    {
		CVaxServerSIP::m_pVaxSIP->RejectRegister(nRegId, 404, "Not Found");
		return;
	}

	CVaxServerSIP::m_pVaxSIP->AuthRegister(nRegId);

	CString sStatus = sUserLogin + CString(" ") + USER_REG_TRYING;
    CVaxServerSIP::m_pVaxSIP->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnRegisterUserSuccess(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserLogin);
	if(pUser == NULL) return;
			
	pUser->OnRegisterSuccess(sUserLogin);
}

void CVaxUser::OnRegisterSuccess(LPCTSTR sUserLogin)
{	
	 if (m_bRegistered == FALSE)
		m_nRegisterUserCount++;

	m_bRegistered = TRUE;

	CString sStatus = sUserLogin + CString(" ") + USER_REG_SUCCESS;
    CVaxServerSIP::m_pVaxSIP->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnRegisterUserFailed(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserLogin);
	if(pUser == NULL) return;

	pUser->OnRegisterFailed(sUserLogin);
}

void CVaxUser::OnRegisterFailed(LPCTSTR sUserLogin)
{
	CString sStatus = sUserLogin + CString(" ") + USER_REG_FAILED;
    CVaxServerSIP::m_pVaxSIP->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnUnRegisterUser(LPCTSTR sUserLogin)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserLogin);
	if(pUser == NULL) return;
	
	pUser->OnUnRegister(sUserLogin);
}

void CVaxUser::OnUnRegister(LPCTSTR sUserLogin)
{	
	if (m_bRegistered == TRUE)
		m_nRegisterUserCount--;

	m_bRegistered = FALSE;

	CString sStatus = sUserLogin + CString(" ") + USER_UNREGISTER;
    CVaxServerSIP::m_pVaxSIP->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

BOOL CVaxUser::AddUser(LPCTSTR sUserName, LPCTSTR sPassword, LPCTSTR sAudioCodecList, BOOL bBlocked, LONG nVoiceTypeId, LPCTSTR sUserPrefix, LPCTSTR sDialPrefix, LPCTSTR sToPeerName)
{
	if(!CVaxServerSIP::m_pVaxSIP->IsInitialized() || CString(sUserName).IsEmpty())
		return FALSE;
	
	RemoveUser(sUserName);
	
	if (!CVaxServerSIP::m_pVaxSIP->AddUser(sUserName, sPassword, sAudioCodecList))
    {
		return FALSE;
	}

	CVaxCallSession::AddUserRoute(sUserName, sUserPrefix, sDialPrefix, sToPeerName);
    CVaxUser::m_pUserList->AddUser(sUserName, bBlocked, nVoiceTypeId);

    CStoreUser::AddUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::RemoveUser(LPCTSTR sUserName)
{
	if(!CVaxServerSIP::m_pVaxSIP->IsInitialized() || CString(sUserName).IsEmpty())
		return;
		
	CVaxServerSIP::m_pVaxSIP->RemoveUser(sUserName);

	CVaxCallSession::RemoveUserRoute(sUserName);
    CVaxUser::m_pUserList->RemoveUser(sUserName);

    CStoreUser::RemoveUser(sUserName);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::LoadUser()
{
	if(!CVaxServerSIP::m_pVaxSIP->IsInitialized())
		return;

	CStringList objListUsers;
	CStoreUser::GetUserAll(objListUsers);

	if(objListUsers.IsEmpty())
		return;

	CString sUserName, sPassword, sAudioCodecList;
	CString sUserPrefix, sDialPrefix, sToPeerName;
    
	BOOL bBlocked = FALSE;
	LONG nVoiceTypeId = VAX_AUDIO_TYPE_NORMAL;
	    		
	while(!objListUsers.IsEmpty())
	{
		sUserName = objListUsers.RemoveHead();
		
		CStoreUser::GetUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
		AddUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
	}
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

BOOL CVaxUser::IsUserTypeAdmin(LPCTSTR sUserName)
{
	CVaxUser* pUser = m_pUserList->GetUser(sUserName);
    if (pUser == NULL) return FALSE;

	return (pUser->m_nVoiceTypeId == VAX_AUDIO_TYPE_ADMIN);
}

int CVaxUser::GetVoiceTypeId(LPCTSTR sUserName)
{
	CVaxUser* pUser = m_pUserList->GetUser(sUserName);
    if (pUser == NULL) return VAX_AUDIO_TYPE_NORMAL;

	return pUser->m_nVoiceTypeId;
}

LONG CVaxUser::GetRegisteredUserCount()
{
	return CVaxUser::m_nRegisterUserCount;
}

