#include "StdAfx.h"
#include "StoreRegistry.h"
#include "StoreQueue.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

#define SUB_KEY_BUFFER_SIZE		2048
#define QUEUE_ROOT_PATH			"Queues\\"

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreQueue::AddQueue(LPCTSTR sQueueName, LONG nProcessMode, LONG nProcessType, CStringList& objAgentList, LPCTSTR sMusicFileName, LPCTSTR sDialToneFileName)
{
	RemoveQueue(sQueueName);

	if(CString(sQueueName).IsEmpty())
		return FALSE;

	CString sKeyPath;
	sKeyPath.Format("%s%s", QUEUE_ROOT_PATH, sQueueName);

	CStoreRegistry objStore;
	
	if(!objStore.SetTEXT(sKeyPath, "QueueName", sQueueName))
	{
		return FALSE;
	}

	CStoreQueue::AddQueueAgentNameAll(sQueueName, objAgentList);

	CStoreQueue::SetQueueProcessMode(sQueueName, nProcessMode, nProcessType);
    CStoreQueue::SetMediaFiles(sQueueName, sMusicFileName, sDialToneFileName);

	return TRUE;
}

void CStoreQueue::RemoveQueue(LPCTSTR sQueueName)
{
	if(CString(sQueueName).IsEmpty())
		return;

	CString sKeyPath;
	sKeyPath.Format("%s%s", QUEUE_ROOT_PATH, sQueueName);

	CStoreRegistry objStore;
	objStore.DeleteSubKey(sKeyPath);	
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

void CStoreQueue::GetQueueNameAll(CStringList& objList)
{
	CString sKeyPath =  QUEUE_ROOT_PATH;

	CStoreRegistry objStore;
	objStore.EnumSubKeys(sKeyPath, objList);
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreQueue::GetQueue(LPCTSTR sQueueName, LONG& nProcessMode, LONG& nProcessType, CStringList& objAgentList, CString& sMusicFileName, CString& sDialToneFileName)
{
	if(CString(sQueueName).IsEmpty())
		return FALSE;

	CStoreQueue::GetQueueProcessMode(sQueueName, nProcessMode, nProcessType);
	CStoreQueue::GetQueueAgentNameAll(sQueueName, objAgentList);

	CStoreQueue::GetMediaFiles(sQueueName, sMusicFileName, sDialToneFileName);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreQueue::SetMediaFiles(LPCTSTR sQueueName, LPCTSTR sMusicFileName, LPCTSTR sDialToneFileName)
{
	CString sPath = QUEUE_ROOT_PATH + CString(sQueueName);
	
	CStoreRegistry objStore;

	objStore.SetTEXT(sPath, "MusicFileName", sMusicFileName);
	objStore.SetTEXT(sPath, "DialToneFileName", sDialToneFileName);
    
	return TRUE;
}

void CStoreQueue::GetMediaFiles(LPCTSTR sQueueName, CString& sMusicFileName, CString& sDialToneFileName)
{
	CString sPath = QUEUE_ROOT_PATH + CString(sQueueName);

	CStoreRegistry objStore;

	objStore.GetTEXT(sPath, "MusicFileName", sMusicFileName, MUSIC_FILE_QUEUE);
	objStore.GetTEXT(sPath, "DialToneFileName", sDialToneFileName, MUSIC_FILE_DIAL_TONE);
}
                        
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreQueue::SetQueueProcessMode(LPCTSTR sQueueName, LONG nProcessMode, LONG nProcessType)
{
	if(CString(sQueueName).IsEmpty())
		return FALSE;

	CString sKeyPath;
	sKeyPath.Format("%s%s", QUEUE_ROOT_PATH, sQueueName);

	CStoreRegistry objStore;

	objStore.SetINT(sKeyPath, "ProcessMode", nProcessMode);
	objStore.SetINT(sKeyPath, "ProcessType", nProcessType);

	return TRUE;
}

void CStoreQueue::GetQueueProcessMode(LPCTSTR sQueueName, LONG& nProcessMode, LONG& nProcessType)
{
	CString sKeyPath;
	sKeyPath.Format("%s%s", QUEUE_ROOT_PATH, sQueueName);

	CStoreRegistry objStore;

	objStore.GetINT(sKeyPath, "ProcessMode", nProcessMode, VAX_QUEUE_PROCESS_MODE_ROUND_ROBIN);
	objStore.GetINT(sKeyPath, "ProcessType", nProcessType, VAX_QUEUE_PROCESS_TYPE_FREE);
}

 /////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

void CStoreQueue::AddQueueAgentNameAll(LPCTSTR sQueueName, CStringList& objList)
{
	if(CString(sQueueName).IsEmpty())
		return;

	CStoreRegistry objStore;
	POSITION hPos = objList.GetHeadPosition();

	while(hPos != NULL)
	{
		CString sUserName = objList.GetNext(hPos);
		
		CString sKeyPath = QUEUE_ROOT_PATH + CString(sQueueName) + "\\" + sUserName;
		objStore.SetTEXT(sKeyPath, "UserName", sUserName);
	}
}
                
void CStoreQueue::GetQueueAgentNameAll(LPCTSTR sQueueName, CStringList& objList)
{
	objList.RemoveAll();

	CString sKeyPath = QUEUE_ROOT_PATH + CString(sQueueName);

	CStoreRegistry objStore;
	objStore.EnumSubKeys(sKeyPath, objList);
}
                        
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreQueue::IsDataStored()
{
	CStoreRegistry objStore;
	return objStore.IsKeyExist(QUEUE_ROOT_PATH);
}