#include "StdAfx.h"
#include "StoreRegistry.h"
#include "StoreRingGroup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

#define SUB_KEY_BUFFER_SIZE		2048
#define RING_GROUP_ROOT_PATH	"RingGroups\\"

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreRingGroup::AddGroup(LPCTSTR sGroupName, LONG nProcessMode, LONG nProcessType, CStringList& objAgentList)
{
	RemoveGroup(sGroupName);

	if(CString(sGroupName).IsEmpty())
		return FALSE;

	CString sKeyPath;
	sKeyPath.Format("%s%s", RING_GROUP_ROOT_PATH, sGroupName);

	CStoreRegistry objStore;
	
	if(!objStore.SetTEXT(sKeyPath, "RingGroupName", sGroupName))
	{
		return FALSE;
	}

	CStoreRingGroup::AddGroupAgentNameAll(sGroupName, objAgentList);
	CStoreRingGroup::SetGroupProcessMode(sGroupName, nProcessMode, nProcessType);

	return TRUE;
}

void CStoreRingGroup::RemoveGroup(LPCTSTR sGroupName)
{
	if(CString(sGroupName).IsEmpty())
		return;

	CString sKeyPath;
	sKeyPath.Format("%s%s", RING_GROUP_ROOT_PATH, sGroupName);

	CStoreRegistry objStore;
	objStore.DeleteSubKey(sKeyPath);	
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

void CStoreRingGroup::GetGroupNameAll(CStringList& objList)
{
	CString sKeyPath = RING_GROUP_ROOT_PATH;

	CStoreRegistry objStore;
	objStore.EnumSubKeys(sKeyPath, objList);
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreRingGroup::GetGroup(LPCTSTR sGroupName, LONG& nProcessMode, LONG& nProcessType, CStringList& objAgentList)
{
	if(CString(sGroupName).IsEmpty())
		return FALSE;

	CStoreRingGroup::GetGroupProcessMode(sGroupName, nProcessMode, nProcessType);
	CStoreRingGroup::GetGroupAgentNameAll(sGroupName, objAgentList);
		
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreRingGroup::SetGroupProcessMode(LPCTSTR sGroupName, LONG nProcessMode, LONG nProcessType)
{
	if(CString(sGroupName).IsEmpty())
		return FALSE;

	CString sKeyPath;
	sKeyPath.Format("%s%s", RING_GROUP_ROOT_PATH, sGroupName);

	CStoreRegistry objStore;

	objStore.SetINT(sKeyPath, "ProcessMode", nProcessMode);
	objStore.SetINT(sKeyPath, "ProcessType", nProcessType);

	return TRUE;
}

void CStoreRingGroup::GetGroupProcessMode(LPCTSTR sGroupName, LONG& nProcessMode, LONG& nProcessType)
{
	CString sKeyPath;
	sKeyPath.Format("%s%s", RING_GROUP_ROOT_PATH, sGroupName);

	CStoreRegistry objStore;

	objStore.GetINT(sKeyPath, "ProcessMode", nProcessMode, VAX_RING_GROUP_PROCESS_MODE_RING_ALL);
	objStore.GetINT(sKeyPath, "ProcessType", nProcessType, VAX_RING_GROUP_PROCESS_TYPE_FREE);
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

void CStoreRingGroup::AddGroupAgentNameAll(LPCTSTR sGroupName, CStringList& objList)
{
	if(CString(sGroupName).IsEmpty())
		return;

	CStoreRegistry objStore;
	POSITION hPos = objList.GetHeadPosition();

	while(hPos != NULL)
	{
		CString sUserName = objList.GetNext(hPos);
		
		CString sKeyPath = RING_GROUP_ROOT_PATH + CString(sGroupName) + "\\" + sUserName;
		objStore.SetTEXT(sKeyPath, "UserName", sUserName);
	}
}
                
void CStoreRingGroup::GetGroupAgentNameAll(LPCTSTR sGroupName, CStringList& objList)
{
	objList.RemoveAll();

	CString sKeyPath = RING_GROUP_ROOT_PATH + CString(sGroupName);

	CStoreRegistry objStore;
	objStore.EnumSubKeys(sKeyPath, objList);
}
                        
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

BOOL CStoreRingGroup::IsDataStored()
{
	CStoreRegistry objStore;
	return objStore.IsKeyExist(RING_GROUP_ROOT_PATH);
}